/*-
 * Copyright (c) 1999,2000 Joao Cabral (jcnc@dhis.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      DHIS(c)  Dynamic Host Information System Release 4.0
 */

#include "dhid.h"


struct conf_t conf;

void off_nl(unsigned char *s) {

	while(*s!='\0' && *s!='\r' && *s!='\n') s++;
	*s='\0';
}

void update_server(int op,int addr,unsigned char *conffile) {

	FILE *f1,*f2;
	unsigned char str[1024];
	
	f1=fopen(conffile,"r");
	if(f1==NULL) { 
		syslog(LOG_ERR,"failed to update server address");
		return;
	}

	sprintf(str,"/tmp/dhid.%d",getpid());
	f2=fopen(str,"w");
	if(f2==NULL) {
		syslog(LOG_ERR,"failed to update server address");
		fclose(f1);
		return;
	}
	
	while(fgets(str,1024,f1)!=NULL) {
		struct in_addr sa;
		if(!strcmp(line_entry(1,str),"isaddr") && op==CTL4_ISADDR) {
			sa.s_addr=addr;
			fprintf(f2,"\tisaddr\t\t%s\n",inet_ntoa(sa));
 			syslog(LOG_INFO,"Control message updated IS address");
			conf.isa=sa.s_addr;

			continue;
		}		
		if(!strcmp(line_entry(1,str),"mxaddr") && op==CTL4_MXADDR) {
			sa.s_addr=addr;
			fprintf(f2,"\tmx_addr\t\t%s\n",inet_ntoa(sa));
 			syslog(LOG_INFO,"Control message updated relay address");
			conf.mxa=sa.s_addr;
			continue;
		}		
		fprintf(f2,"%s",str);
	}
	fclose(f1);
	fclose(f2);
	sprintf(str,"/tmp/dhid.%d",getpid());
	f1=fopen(str,"r");
	if(f1==NULL) { 
		syslog(LOG_ERR,"failed to update server address");
		return;
	}
	f2=fopen(conffile,"w");
	if(f2==NULL) {
		syslog(LOG_ERR,"failed to update server address");
		fclose(f1);
		return;
	}
	while(fgets(str,1024,f1)!=NULL) 
		fprintf(f2,"%s",str);
	fclose(f1);
	fclose(f2);
	sprintf(str,"/tmp/dhid.%d",getpid());
	unlink(str);
	return;
}


void read_conf(unsigned char *file) {

	FILE *fp;
	unsigned char line[1024];
	unsigned char str[64];

#ifdef	QRC
	unsigned char keyp[1024],keyq[1024];
	keyp[0]='\0';
	keyq[0]='\0';
#endif

	conf.id=conf.isa=conf.mxa=0;
	conf.isp=DEF_ISPORT;
	conf.atype=0;
	conf.sid=0;

	conf.pass[0]=conf.host[0]='\0';

	fp=fopen(file,"r");
	if(fp==NULL) { 
		syslog(LOG_ERR,"failed to read %s",file);
		exit(1);
	}

	while(fgets(line,1024,fp)!=NULL) {
	off_nl(line);
	if(!strcmp(line_entry(1,line),"hostid"))
		conf.id=atoi(line_entry(2,line));

	if(!strcmp(line_entry(1,line),"hostname"))
		strcpy(conf.host,line_entry(2,line));

	if(!strcmp(line_entry(1,line),"hostpass")) {
		strcpy(conf.pass,line_entry(2,line));
		conf.atype=APASS;
	}

	if(!strcmp(line_entry(1,line),"isaddr")) {
		strcpy(str,line_entry(2,line));
		if(isdigit(str[0])) conf.isa=inet_addr(str);
	}

	if(!strcmp(line_entry(1,line),"mxaddr")) {
		strcpy(str,line_entry(2,line));
		if(isdigit(str[0])) conf.mxa=inet_addr(str);
	}

	if(!strcmp(line_entry(1,line),"authp")) {
#ifdef	QRC
		strcat(keyp,line_entry(2,line));
		conf.atype=AQRC;
#else
		syslog(LOG_ERR,"QRC authentication not available in this client");
		syslog(LOG_ERR,"Please compile QRC support. Exiting ...");
		exit(255);
#endif
	}

	if(!strcmp(line_entry(1,line),"authq")) {
#ifdef	QRC
		strcat(keyq,line_entry(2,line));
		conf.atype=AQRC;
#else
		syslog(LOG_ERR,"QRC authentication not available in this client");
		syslog(LOG_ERR,"Please compile QRC support. Exiting ...");
		exit(255);
#endif
	}
	

	
	}
	fclose(fp);

	if(conf.atype==0) {
		syslog(LOG_ERR,"Failed to read authentication key");
		exit(255);
	}

	if(conf.atype==APASS && conf.pass[0]=='\0') {
		syslog(LOG_ERR,"Failed to read text authentication key");
		exit(255);
	}

#ifdef	QRC
	if(conf.atype==AQRC && (keyp[0]=='\0' || keyq[0]=='\0')) {
		syslog(LOG_ERR,"Failed to read QRC authentication keys");
		exit(255);
	}
#endif

	if(conf.id==0) {	
		syslog(LOG_ERR,"Failed to read host id");
		exit(255);
	}

	if(conf.isa==0) {	
		syslog(LOG_ERR,"Failed to read remote server address");
		exit(255);
	}

#ifdef	QRC
	if(conf.atype==AQRC) {
	mpz_init(conf.authp);
	mpz_init(conf.authq);
	mpz_set_str(conf.authp,keyp,10);
	mpz_set_str(conf.authq,keyq,10);
	}
#endif

	return;
}



unsigned char *line_entry(int idx,unsigned char *buff) {

	static unsigned char b2[1024],*pb2;
	int i;

	idx--;
	b2[0]='\0';
	pb2=b2;

	while((*buff==' ' || *buff=='\t') && *buff!='\0' && *buff!='\n')
		buff++;
	if(*buff=='\0' || *buff=='\n') return(b2);
		
	for(i=0;i<idx;i++) {
		while(*buff!=' ' && *buff!='\t' && *buff!='\0' &&
			*buff!='\n') buff++;
		if(*buff=='\0' || *buff=='\n') return(b2);
		while((*buff==' ' || *buff=='\t') && *buff!='\0' && *buff!='\n')
			buff++;
		if(*buff=='\0' || *buff=='\n') return(b2);
	}
	while(*buff!=' ' && *buff!='\t' && *buff!='\0' && *buff!='\n') {
		*pb2 = *buff;
		buff++;
		pb2++;
	}
	*pb2 = '\0';
	return(b2);
}

